import logging

import django.contrib.auth.password_validation as validators
from django.core import exceptions
from rest_framework import serializers

from core import models

from .generic import DynamicFieldsModelSerializer

log = logging.getLogger(__name__)


class ExamSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = models.ExamType
        fields = ('pk', 'module_reference', 'total_score',
                  'pass_score', 'pass_only',)


class TestSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = models.Test
        fields = ('pk', 'name', 'label', 'annotation')


class SubmissionTypeListSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = models.SubmissionType
        fields = ('pk', 'name', 'full_score')


class SubmissionTypeSerializer(SubmissionTypeListSerializer):

    class Meta:
        model = models.SubmissionType
        fields = ('pk',
                  'name',
                  'full_score',
                  'description',
                  'solution',
                  'programming_language')


class UserAccountSerializer(DynamicFieldsModelSerializer):

    def validate(self, data):
        password = data.get('password')

        try:
            if password is not None:
                validators.validate_password(password=password,
                                             user=self.instance)
        except exceptions.ValidationError as err:
            raise serializers.ValidationError({'password': list(err.messages)})
        return data

    class Meta:
        model = models.UserAccount
        fields = ('pk', 'username', 'role', 'is_admin', 'password')
        read_only_fields = ('pk', 'username', 'role', 'is_admin')
        extra_kwargs = {'password': {'write_only': True}}
