from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework_csv import renderers

from core.models import StudentInfo, SubmissionType
from core.permissions import IsReviewer


class SemicolonCSVRenderer(renderers.CSVRenderer):
    writer_opts = {
        'delimiter': ';',
    }


class StudentCSVExport(APIView):
    renderer_classes = (SemicolonCSVRenderer, )
    permission_classes = (IsReviewer, )

    def get_renderer_context(self):
        context = super().get_renderer_context()
        context['header'] = ('Matrikel', 'Name', 'Sum',
                             *SubmissionType.objects.values_list('name',
                                                                 flat=True))
        context['delimiter'] = ';'
        return context

    def finalize_response(self, request, response, *args, **kwargs):
        response['Content-Disposition'] = \
            "attachment; filename=%s" % 'results.csv'
        return super().finalize_response(request, response, *args, **kwargs)

    def get(self, request, format=None):
        content = [{'Matrikel': student.matrikel_no,
                    'Name': student.user.fullname,
                    'Sum': student.overall_score,
                    **student.score_per_submission()
                    } for student
                   in StudentInfo.get_annotated_score_submission_list()]

        return Response(content)
