import { mut } from './mutations'
import { authMut } from '@/store/modules/authentication'
import { subNotesMut } from '@/store/modules/submission-notes'
import * as api from '@/api'
import router from '@/router/index'
import { handleError } from '@/util/helpers'

const actions = {
  async getExamTypes ({commit, dispatch}) {
    try {
      const examTypes = await api.fetchExamType({})
      commit(mut.SET_EXAM_TYPES, examTypes)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch exam types')
    }
  },
  async updateSubmissionTypes ({commit, dispatch}, fields = []) {
    try {
      const submissionTypes = await api.fetchSubmissionTypes(fields)
      submissionTypes.forEach(type => {
        commit(mut.UPDATE_SUBMISSION_TYPE, type)
      })
    } catch (err) {
      handleError(err, dispatch, 'Unable to get submission types')
    }
  },
  async getStudents ({commit, dispatch}, opt = {studentPks: [], fields: []}) {
    try {
      if (opt.studentPks.length === 0) {
        const students = await api.fetchAllStudents()
        commit(mut.SET_STUDENTS, students)
        return students
      } else {
        const students = await Promise.all(
          opt.studentPks.map(pk => api.fetchStudent({
            pk,
            fields: opt.fields
          }))
        )
        students.forEach(student => commit(mut.SET_STUDENT, student))
        return students
      }
    } catch (err) {
      console.log(err)
      handleError(err, dispatch, 'Unable to fetch student data')
    }
  },
  async getTutors ({commit, dispatch}) {
    try {
      const tutors = await api.fetchAllTutors()
      commit(mut.SET_TUTORS, tutors)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch tutor data')
    }
  },
  async getAllFeedback ({commit, dispatch}, fields = []) {
    try {
      const feedback = await api.fetchAllFeedback({fields})
      commit(mut.SET_ALL_FEEDBACK, feedback)
      commit(mut.MAP_FEEDBACK_OF_SUBMISSION_TYPE)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch feedback history')
    }
  },
  async getFeedback ({commit, dispatch}, {ofSubmission}) {
    try {
      const feedback = await api.fetchFeedback({ofSubmission})
      commit(mut.SET_FEEDBACK, feedback)
      return feedback
    } catch (err) {
      handleError(err, dispatch, `Unable to fetch feedback ${ofSubmission}`)
    }
  },
  async getSubmissionFeedbackTest ({commit, dispatch}, {pk}) {
    try {
      const submission = await api.fetchSubmissionFeedbackTests({pk})
      commit(mut.SET_SUBMISSION, submission)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch submission')
    }
  },
  async getStatistics ({commit, dispatch}, opt) {
    try {
      const statistics = await api.fetchStatistics(opt)
      commit(mut.SET_STATISTICS, statistics)
    } catch (err) {
      handleError(err, dispatch, 'Unable to fetch statistics')
    }
  },
  async deleteComment ({commit, dispatch}, comment) {
    try {
      await api.deleteComment(comment)
    } catch (err) {
      handleError(err, dispatch, `Unable to delete comment: ${comment.text}`)
    }
  },
  async patchComment ({commit, dispatch}, comment) {
    try {
      return await api.patchComment(comment)
    } catch (err) {
      handleError(err, dispatch, `Unable to patch comment: ${comment.text}`)
    }
  },
  logout ({ commit, getters, state }, message = '') {
    if (getters.isStudent) {
      // change active to false when user logs out
      api.changeActiveForUser(state.authentication.user.pk, false)
    }
    commit('RESET_STATE')
    commit('submissionNotes/' + subNotesMut.RESET_STATE)
    commit(authMut.SET_MESSAGE, message)
    router.push({name: 'login'})
    router.go()
  }
}

export default actions
