export const namespace = 'feedbackSearchOptions'

export const feedbackSearchOptsMut = Object.freeze({
  SET_SHOW_FINAL: 'SET_SHOW_FINAL',
  SET_SEARCH_OTHER_USER_COMMENTS: 'SET_SEARCH_OTHER_USER_COMMENTS',
  SET_CASE_SENSITIVE: 'SET_CASE_SENSITIVE',
  SET_USE_REGEX: 'SET_USE_REGEX',
  SET_FILTER_BY_TUTORS: 'SET_FILTER_BY_TUTORS',
  SET_FILTER_BY_STAGE: 'SET_FILTER_BY_STAGE'
})

function initialState () {
  return {
    showFinal: false,
    searchOtherUserComments: false,
    caseSensitive: false,
    useRegex: false,
    filterByTutors: [],
    filterByStage: ''
  }
}

const mapStageDisplayToApiString = {
  'Initial feedback': 'feedback-creation',
  'Validation': 'feedback-validation'
}

const feedbackSearchOptions = {
  namespaced: true,
  state: initialState(),
  getters: {
    stageFilterString (state) {
      return mapStageDisplayToApiString[state.filterByStage] || 'all'
    }
  },
  mutations: {
    [feedbackSearchOptsMut.SET_SHOW_FINAL]: (state, val) => { state.showFinal = val },
    [feedbackSearchOptsMut.SET_SEARCH_OTHER_USER_COMMENTS]: (state, val) => {
      state.searchOtherUserComments = val
    },
    [feedbackSearchOptsMut.SET_CASE_SENSITIVE]: (state, val) => { state.caseSensitive = val },
    [feedbackSearchOptsMut.SET_USE_REGEX]: (state, val) => { state.useRegex = val },
    [feedbackSearchOptsMut.SET_FILTER_BY_TUTORS]: (state, val) => { state.filterByTutors = val },
    [feedbackSearchOptsMut.SET_FILTER_BY_STAGE]: (state, val) => { state.filterByStage = val }
  }
}

export default feedbackSearchOptions
