import {fetchAllFeedback, fetchAllAssignments} from '@/api'
import {objectifyArray} from '@/util/helpers'

export const feedbackTableMut = Object.freeze({
  SET_FEEDBACK_HISTORY: 'SET_FEEDBACK_HISTORY',
  ADD_ASSIGNMENTS_INFO: 'ADD_ASSIGNMENTS_INFO',
  SET_FEEDBACK_OF_SUBMISSION_TYPE: 'SET_FEEDBACK_OF_SUBMISSION_TYPE',
  MAP_FEEDBACK_HIST_OF_SUBMISSION_TYPE: 'MAP_FEEDBACK_HIST_OF_SUBMISSION_TYPE',
  RESET_STATE: 'RESET_STATE'
})

function initialState () {
  return {
    feedbackHist: {}
  }
}

const feedbackTable = {
  state: initialState(),
  mutations: {
    [feedbackTableMut.SET_FEEDBACK_HISTORY]: (state, val) => {
      state.feedbackHist = objectifyArray(val, 'of_submission')
    },
    [feedbackTableMut.ADD_ASSIGNMENTS_INFO]: (state, assignments) => {
      for (const assignment of assignments) {
        const feedback = state.feedbackHist[assignment.submission]
        feedback.history = {
          ...feedback.history,
          [assignment.stage]: {
            owner: assignment.owner,
            is_done: assignment.is_done
          }
        }
      }
    },
    [feedbackTableMut.SET_FEEDBACK_OF_SUBMISSION_TYPE] (state, {feedback, type}) {
      state.feedbackHist[feedback.of_submission].of_submission_type = type.name
    },
    [feedbackTableMut.RESET_STATE]: (state) => { Object.assign(state, initialState()) }
  },
  actions: {
    mapFeedbackHistOfSubmissionType ({getters, commit, state}) {
      for (const feedback of Object.values(state.feedbackHist)) {
        const type = getters.getSubmissionType(feedback.of_submission_type)
        commit(feedbackTableMut.SET_FEEDBACK_OF_SUBMISSION_TYPE, {feedback, type})
      }
    },
    async getFeedbackHistory ({getters, commit, dispatch}) {
      let data = []
      data.push(fetchAllFeedback())
      if (getters.isReviewer) {
        data.push(fetchAllAssignments())
      }
      Promise.all(data).then(([feedback, assignments]) => {
        commit(feedbackTableMut.SET_FEEDBACK_HISTORY, feedback)
        dispatch('mapFeedbackHistOfSubmissionType')
        if (assignments) {
          commit(feedbackTableMut.ADD_ASSIGNMENTS_INFO, assignments)
        }
      })
    }
  }
}

export default feedbackTable
