import {fetchStudentSelfData, fetchStudentSubmissions} from '@/api'

function initialState () {
  return {
    studentName: '',
    exam: {},
    submissionsForList: [],
    submissionData: {},
    visited: {},
    loaded: false
  }
}

export const studentPageMut = Object.freeze({
  SET_STUDENT_NAME: 'SET_STUDENT_NAME',
  SET_EXAM: 'SET_EXAM',
  SET_SUBMISSION_TYPES: 'SET_SUBMISSION_TYPES',
  SET_SUBMISSIONS_FOR_LIST: 'SET_SUBMISSIONS_FOR_LIST',
  SET_FULL_SUBMISSION_DATA: 'SET_FULL_SUBMISSION_DATA',
  SET_VISITED: 'SET_VISITED',
  SET_LOADED: 'SET_LOADED',
  RESET_STATE: 'RESET_STATE'
})

const studentPage = {
  state: initialState(),
  mutations: {
    [studentPageMut.SET_STUDENT_NAME] (state, name) {
      state.studentName = name
    },
    [studentPageMut.SET_EXAM] (state, exam) {
      state.exam = exam
    },
    [studentPageMut.SET_SUBMISSION_TYPES] (state, submissionTypes) {
      state.submissionTypes = submissionTypes
    },
    [studentPageMut.SET_SUBMISSIONS_FOR_LIST] (state, submissions) {
      state.submissionsForList = submissions
    },
    /**
     * Reduces the array submissionData returned by the /api/student-submissions
     * into an object where the keys are the SubmissionType id's and the values
     * the former array elements. This is done to have direct access to the data
     * via the SubmissionType id.
     */
    [studentPageMut.SET_FULL_SUBMISSION_DATA] (state, submissionData) {
      state.submissionData = submissionData.reduce((acc, cur, index) => {
        acc[cur.type.pk] = cur
        return acc
      }, {})
    },
    [studentPageMut.SET_VISITED] (state, visited) {
      state.visited = { ...state.visited, [visited.index]: visited.visited }
    },
    [studentPageMut.SET_LOADED] (state, loaded) {
      state.loaded = loaded
    },
    [studentPageMut.RESET_STATE] (state) {
      Object.assign(state, initialState())
    }
  },
  actions: {

    async getStudentData (context) {
      try {
        const studentData = await fetchStudentSelfData()
        context.commit(studentPageMut.SET_STUDENT_NAME, studentData.name)
        context.commit(studentPageMut.SET_EXAM, studentData.exam)
        context.commit(studentPageMut.SET_SUBMISSIONS_FOR_LIST, studentData.submissions)
        context.commit(studentPageMut.SET_LOADED, true)
      } catch (e) {
        console.log(e)
      }
    },

    async getStudentSubmissions (context) {
      try {
        const submissions = await fetchStudentSubmissions()
        context.commit(studentPageMut.SET_FULL_SUBMISSION_DATA, submissions)
      } catch (e) {
        console.log(e)
      }
    }
  }
}

export default studentPage
