import Vue from 'vue'

import {initialState} from '@/store/store'

export const mut = Object.freeze({
  SET_LAST_INTERACTION: 'SET_LAST_INTERACTION',
  SET_EXAM_TYPES: 'SET_EXAM_TYPES',
  SET_STUDENTS: 'SET_STUDENTS',
  SET_STUDENT: 'SET_STUDENT',
  SET_STUDENT_MAP: 'SET_STUDENT_MAP',
  SET_TUTORS: 'SET_TUTORS',
  SET_SUBMISSION: 'SET_SUBMISSION',
  SET_ALL_FEEDBACK: 'SET_ALL_FEEDBACK',
  SET_STATISTICS: 'SET_STATISTICS',
  SET_FEEDBACK: 'SET_FEEDBACK',
  MAP_FEEDBACK_OF_SUBMISSION_TYPE: 'MAP_FEEDBACK_OF_SUBMISSION_TYPE',
  UPDATE_SUBMISSION_TYPE: 'UPDATE_SUBMISSION_TYPE',
  UPDATE_SUBMISSION: 'UPDATE_SUBMISSION',
  RESET_STATE: 'RESET_STATE'
})

const mutations = {
  [mut.SET_EXAM_TYPES] (state, examTypes) {
    state.examTypes = examTypes.reduce((acc, curr) => {
      acc[curr.pk] = curr
      return acc
    }, {})
  },
  [mut.SET_SUBSCRIPTIONS] (state, subscriptions) {
    state.subscriptions = subscriptions.reduce((acc, curr) => {
      acc[curr['pk']] = curr
      return acc
    }, {})
    for (let subscription of Object.values(subscriptions)) {
      for (let assignment of subscription.assignments) {
        if (assignment.feedback) {
          Vue.set(assignment.feedback, 'feedback_stage_for_user', subscription.feedback_stage)
        }
      }
    }
  },
  [mut.SET_SUBSCRIPTION] (state, subscription) {
    Vue.set(state.subscriptions, subscription.pk, subscription)
    for (let assignment of subscription.assignments) {
      if (assignment.feedback) {
        Vue.set(assignment.feedback, 'feedback_stage_for_user', subscription.feedback_stage)
      }
    }
  },
  [mut.SET_STUDENTS] (state, students) {
    state.students = students.reduce((acc, curr) => {
      acc[curr.pk] = mapStudent(curr, state.studentMap)
      return acc
    }, {})
  },
  [mut.SET_STUDENT] (state, student) {
    Vue.set(state.students, student.pk, mapStudent({
      ...state.students[student.pk],
      ...student
    }, state.studentMap))
  },
  [mut.SET_STUDENT_MAP] (state, map) {
    state.studentMap = map
  },
  [mut.SET_TUTORS] (state, tutors) {
    state.tutors = tutors
  },
  [mut.SET_SUBMISSION] (state, submission) {
    Vue.set(state.submissions, submission.pk, submission)
  },
  [mut.SET_ALL_FEEDBACK] (state, feedbackList) {
    state.feedback = feedbackList.reduce((acc, curr) => {
      acc[curr.pk] = curr
      return acc
    }, {})
  },
  [mut.SET_FEEDBACK] (state, feedback) {
    Vue.set(state.feedback, feedback.pk, {
      ...state.feedback[feedback.pk],
      ...feedback,
      of_submission_type: state.submissionTypes[feedback['of_submission_type']]
    })
  },
  [mut.SET_STATISTICS] (state, statistics) {
    state.statistics = {
      ...state.statistics,
      ...statistics
    }
  },
  [mut.MAP_FEEDBACK_OF_SUBMISSION_TYPE] (state) {
    Object.values(state.feedback).forEach(feedback => {
      const submissionType = state.submissionTypes[feedback['of_submission_type']]
      if (submissionType) {
        feedback['of_submission_type'] = submissionType
      }
    })
  },
  [mut.UPDATE_SUBMISSION_TYPE] (state, submissionType) {
    const updatedSubmissionType = {
      ...state.submissionTypes[submissionType.pk],
      ...submissionType
    }
    Vue.set(state.submissionTypes, submissionType.pk, updatedSubmissionType)
  },
  [mut.UPDATE_SUBMISSION] (state, {submissionPk, payload, key}) {
    Vue.set(state.submissions[submissionPk], key, payload)
  },
  [mut.SET_LAST_INTERACTION] (state) {
    state.lastAppInteraction = Date.now()
  },
  [mut.RESET_STATE] (state) {
    Object.assign(state, initialState())
  }
}

function mapStudent (student, map) {
  if (Object.keys(map).length > 0) {
    return {
      ...student,
      ...map[student.matrikel_no]
    }
  }
  return student
}

export default mutations
