import Vuex from 'vuex'
import Vue from 'vue'
import createPersistedState from 'vuex-persistedstate'

import studentPage from './modules/student-page'
import submissionNotes from './modules/submission-notes'
import authentication from './modules/authentication'
import ui from './modules/ui'
import subscriptions from './modules/subscriptions'
import feedbackTable from './modules/feedback_list/feedback-table'
import feedbackSearchOptions from './modules/feedback_list/feedback-search-options'

import actions from './actions'
import getters from './getters'
import mutations from '@/store/mutations'
import {lastInteraction} from '@/store/plugins/lastInteractionPlugin'

Vue.use(Vuex)

export function initialState () {
  return {
    lastAppInteraction: Date.now(),
    examTypes: {},
    submissionTypes: {},
    submissions: {},
    feedback: {},
    students: {},
    studentMap: {}, // is used to map obfuscated student data back to the original
    statistics: {
      submission_type_progress: []
    },
    tutors: []
  }
}

export const persistedStateKey = 'grady'

const store = new Vuex.Store({
  strict: process.env.NODE_ENV === 'development',
  modules: {
    authentication,
    studentPage,
    submissionNotes,
    ui,
    subscriptions,
    feedbackTable,
    feedbackSearchOptions
  },
  plugins: [
    createPersistedState({
      key: persistedStateKey,
      storage: window.sessionStorage,
      // authentication.token is manually saved since using it with this plugin caused issues
      // when manually reloading the page
      paths: Object.keys(initialState()).concat(
        ['ui', 'studentPage', 'submissionNotes', 'feedbackSearchOptions', 'subscriptions',
          'authentication.user', 'authentication.jwtTimeDelta',
          'authentication.tokenCreationTime'])
    }),
    lastInteraction],
  actions,
  getters,
  mutations,
  state: initialState()
})

export default store
